=begin
(c) TIG 2009
material-make_unique.rb adds a new Method to Sketchup's Material Class - 
  material.make_unique
It makes a cloned of the material from the original and gives it a 
new name, incrementing the 'name' with #n at the end.
Returns a unique new material.

v1.0 1st issue 20090904

=end
require 'sketchup.rb'
class Sketchup::Material
  def make_unique()
    model=Sketchup.active_model
    ents=model.active_entities
    mats=model.materials
    ### get self details
    name=self.display_name
    alpha=self.alpha
    color=self.color
    texture=self.texture
    attrdicts=self.attribute_dictionaries
    ### make new material to match self
    names=[];mats.each{|m|names.push(m.display_name)}
    num=1; new_name=name
    while names.include?(new_name)
      new_name=new_name+"#"+num.to_s; num+=1
    end#
    newMat=mats.add(new_name)
    newMat.alpha=alpha
    newMat.color=color
    attrdicts.each{|attrdict|
      name=attrdict.name
      attrdict.each_pair{|key,value|newMat.set_attribute(name,key,value)}
    }if attrdicts
    if texture
      size=texture.height
      filename=texture.filename
      if filename.include?("\\") or filename.include?("/")#PC || Mac
        if FileTest.exist?(filename)### it's image was set 'manually'
          newMat.texture=filename.tr("\\","/")
          newMat.texture.size=size
        else### was temp, so not found, re-save and use it's name
          filename=filename.tr("\\","/").split("/").last
          tdir=ENV["TEMP"]
          tdir=ENV["TMPDIR"]if not tdir ### it's a Mac v1.1
          tdir=tdir.tr("\\","/")+"/"
          filepath=tdir+filename
          group=ents.add_group
          group.material=self
          tw=Sketchup.create_texture_writer
          tw.load(group)
          tw.write(group,filepath)
          newMat.texture=filepath
          newMat.texture.size=size
          File.delete(filepath)
        end#if
      else ### it's a bound image
        tdir=ENV["TEMP"]
        tdir=ENV["TMPDIR"]if not tdir ### it's a Mac v1.1
        tdir=tdir.tr("\\","/")+"/"
        filepath=tdir+filename
        group=ents.add_group
        group.material=self
        tw=Sketchup.create_texture_writer
        tw.load(group)
        tw.write(group,filepath)
        newMat.texture=filepath
        newMat.texture.size=size
        File.delete(filepath)
      end#if
    end#if
    return newMat
  end#def
end#class
###
### Example of use...
### make material of selected faces unique
### type makefacesmaterialsunique in the Ruby Console
def makefacesmaterialsunique()
  require 'material-make_unique.rb'
  model=Sketchup.active_model
  ss=model.selection
  faces=[]
  ss.each{|e|faces<<e if e.class==Sketchup::Face}
  if not faces[0]
    puts "No Faces in Selection"
    return nil
  end#if
  count=0
  faces.each{|face|
    mat=face.material
    if mat
      unique_mat=mat.make_unique
      face.material=unique_mat
      count+=1
    end#if
  }
  puts count.to_s+" Face Material made unique." if count==1
  puts count.to_s+" Face Materials made unique." if count!=1
end#def